# Generated by Django 3.0.2 on 2020-01-06 21:56

import autoslug.fields
from django.db import migrations, models
import django.db.models.deletion
import mptt.fields


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Category',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=50)),
                ('slug', autoslug.fields.AutoSlugField(editable=False, populate_from='name')),
                ('level', models.IntegerField(choices=[(1, 'voivodeship'), (2, 'county'), (3, 'community')], db_index=True)),
            ],
            options={
                'verbose_name': 'Category',
                'verbose_name_plural': 'Categories',
            },
        ),
        migrations.CreateModel(
            name='JednostkaAdministracyjna',
            fields=[
                ('id', models.CharField(max_length=7, primary_key=True, serialize=False)),
                ('name', models.CharField(max_length=36, verbose_name='Name')),
                ('slug', autoslug.fields.AutoSlugField(editable=False, populate_from='name', unique=True)),
                ('updated_on', models.DateField(verbose_name='Updated date')),
                ('active', models.BooleanField(default=False)),
                ('lft', models.PositiveIntegerField(editable=False)),
                ('rght', models.PositiveIntegerField(editable=False)),
                ('tree_id', models.PositiveIntegerField(db_index=True, editable=False)),
                ('level', models.PositiveIntegerField(editable=False)),
                ('category', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='teryt_tree.Category')),
                ('parent', mptt.fields.TreeForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='children', to='teryt_tree.JednostkaAdministracyjna')),
            ],
            options={
                'verbose_name': 'Unit of administrative division',
                'verbose_name_plural': 'Units of administrative division',
            },
        ),
        migrations.CreateModel(
            name='SIMC',
            fields=[
                ('id', models.CharField(max_length=7, primary_key=True, serialize=False)),
                ('name', models.CharField(max_length=100)),
                ('updated_on', models.DateField(verbose_name='Updated date')),
                ('sym_pod', models.ForeignKey(blank=True, on_delete=django.db.models.deletion.CASCADE, to='teryt_tree.SIMC')),
                ('terc', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='teryt_tree.JednostkaAdministracyjna')),
            ],
        ),
    ]



import django_filters
from django.shortcuts import get_object_or_404
from django.utils.translation import ugettext_lazy as _

try:
    from django_filters import rest_framework as filters
except ImportError:  # Back-ward compatible for django-rest-framework<3.7
    from rest_framework import filters
from rest_framework import viewsets
from teryt_tree.models import JednostkaAdministracyjna
from teryt_tree.rest_framework_ext.serializers import \
    JednostkaAdministracyjnaSerializer


def custom_area_filter(queryset, _, value):
    if not value:
        return queryset
    return queryset.area(get_object_or_404(JednostkaAdministracyjna, pk=value))


class JednostkaAdministracyjnaFilter(filters.FilterSet):
    area = django_filters.CharFilter(
        method=custom_area_filter,
        label=_("Area")
    )

    class Meta:
        model = JednostkaAdministracyjna
        fields = ['name', 'category', 'category__level', 'area']


class JednostkaAdministracyjnaViewSet(viewsets.ModelViewSet):
    queryset = (JednostkaAdministracyjna.objects.
                select_related('category').
                prefetch_related('children').
                all())
    serializer_class = JednostkaAdministracyjnaSerializer
    filter_backends = (filters.DjangoFilterBackend,)
    filter_class = JednostkaAdministracyjnaFilter



# -*- coding: utf-8 -*-
import argparse

from tqdm import tqdm
from django.core.management.base import BaseCommand, CommandError
from django.db import transaction
from functools import lru_cache

try:
    from lxml import etree
except ImportError:
    raise CommandError('Missing dependency. Please, install lxml>=2.3.3')

from teryt_tree.models import Category, JednostkaAdministracyjna


class Command(BaseCommand):
    help = 'Creates a data in database base on TERC.xml file.'
    PARENT_REDUCE = {2: 0,
                     4: 2,
                     7: 4}
    LEVEL_REDUCE = {2: 1,
                    4: 2,
                    7: 3}
    FORMAT_MAP = {'nazwa_dod': 'nazdod'}

    def add_arguments(self, parser):
        parser.add_argument('--input', type=argparse.FileType('r'), nargs='?', help="Input XML-file")
        parser.add_argument('--old-format', dest='old_format', action='store_true',
                            help="Use format data of teryt.stat.gov.pl")
        parser.add_argument('--no-progress', dest='no_progress', action='store_false')

    def to_object(self, row, old_format):
        if old_format:
            data = {x.get('name').lower(): x.text for x in row}
        else:
            data = {self.FORMAT_MAP.get(x.tag.lower(), x.tag.lower()): (x.text or "").strip()
                    for x in row}
        obj = JednostkaAdministracyjna()
        obj.active = True
        obj.id = "".join(data.get(x) or '' for x in ('woj', 'pow', 'gmi', 'rodz'))
        index = len(obj.pk)
        if len(obj.id) > 2:
            obj.parent_id = obj.id[:self.PARENT_REDUCE[index]]
        obj.name = data['nazwa'].title()
        obj.updated_on = data['stan_na']
        obj.category = self.get_genre(data['nazdod'], self.LEVEL_REDUCE[index])
        return obj

    @lru_cache()
    def get_genre(self, name, level):
        obj, _ = Category.objects.get_or_create(name=name, defaults={'level': level})
        return obj

    def handle(self, no_progress, input, old_format, *args, **options):
        root = etree.parse(input)
        self.stdout.write("Importing started. This may take a few seconds. Please wait a moment.\n")
        with transaction.atomic():
            with JednostkaAdministracyjna.objects.delay_mptt_updates():
                for row in self.get_iter(root.iter('row'), no_progress):
                    item = self.to_object(row, old_format)
                    item.save()

    def get_iter(self, items, no_progress):
        return tqdm(items) if no_progress else items



import django_filters

from teryt_tree.models import JednostkaAdministracyjna


class JSTModelChoice(django_filters.ModelChoiceFilter):
    model = JednostkaAdministracyjna

    @staticmethod
    def filter__area(queryset, value):
        return queryset.filter(jst__tree_id=value.tree_id, jst__lft__range=(value.lft, value.rght))

    def get_queryset(self):
        qs = self.model.objects.select_related('category')
        if self.level:
            qs = qs.filter(category__level=self.level)
        return qs

    def __init__(self, level=None, *args, **kwargs):
        self.level = level
        kwargs['method'] = self.filter__area
        kwargs['queryset'] = self.get_queryset()
        super().__init__(*args, **kwargs)

    def limit_parent(self, value):
        qs = self.field.queryset
        self.field.queryset = qs.filter(parent=value)



# -*- coding: utf-8 -*-
import argparse
from datetime import datetime

from django.core.management.base import BaseCommand, CommandError
from functools import lru_cache
from tqdm import tqdm

try:
    from lxml import etree
except ImportError:
    raise CommandError('Missing dependency. Please, install lxml>=2.3.3')

from teryt_tree.models import SIMC, JednostkaAdministracyjna


class Command(BaseCommand):
    help = 'Creates a data in database base on SIMC.xml file.'

    def add_arguments(self, parser):
        parser.add_argument('--input', nargs='?', type=argparse.FileType('r'),
                            help="Input XML-file")
        parser.add_argument('--old-format', dest='old_format', action='store_true',
                            help="Use format data of teryt.stat.gov.pl")
        parser.add_argument('--no-progress', dest='no_progress', action='store_false')

    @lru_cache()
    def get_terc(self, terc_id):
        return JednostkaAdministracyjna.objects.get(id=terc_id)

    def to_object(self, row, old_format):
        if old_format:
            data = {x.get('name').lower(): x.text for x in row}
        else:
            data = {x.tag.lower(): (x.text or "").strip() for x in row}

        terc_id = "".join(data.get(x, '') or '' for x in ('woj', 'pow', 'gmi', 'rodz_gmi'))
        terc = self.get_terc(terc_id)
        return SIMC(id=data['sym'],
                    sym_pod_id=data['sympod'],
                    terc=terc,
                    updated_on=datetime.strptime(data['stan_na'], '%Y-%m-%d'))

    def handle(self, input, no_progress, old_format, *args, **options):
        root = etree.parse(input)
        self.stdout.write("Importing started. This may take a few seconds. Please wait a moment.\n")
        SIMC.objects.bulk_create(self.to_object(x, old_format)
                                 for x in self.get_iter(root.iter('row'), no_progress))

    def get_iter(self, queryset, no_progress):
        return tqdm(queryset) if no_progress else queryset
